// ========================= AtxmegaxxxA1 ADC  α׷ ==================
//  CPU : ATmegaxxxA1
//     ¥ : 2010-
//      : Ver1.0
//    : www.cpuplaza.com
// Ϸ : CodeVisionAVR 2.04.6
// ----------------------------------------------------------------------------
// ⺻ 
#include <io.h>							// CPU  

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdbool.h>
#include <delay.h>

//  
#include "adc_drv.h"						// ADC 

// ======================== ADC CalibrationValues Load ========================
// [μ] ADC_t * adc
// Item1: ADC Port (ADCA,ADCB)
// ----------------------------------------------------------------------------
void ADC_CalibrationValues_Load(ADC_t *adc)
{
	if(&ADCA == adc){					// Get ADCACAL0 from production signature.
		(adc)->CALL = SP_ReadCalibrationByte(PROD_SIGNATURES_START+ADCACAL0_offset );
		(adc)->CALH = SP_ReadCalibrationByte(PROD_SIGNATURES_START+ADCACAL1_offset );
	}
	else{							// Get ADCBCAL0 from production signature
		(adc)->CALL = SP_ReadCalibrationByte(PROD_SIGNATURES_START+ADCBCAL0_offset );
		(adc)->CALH = SP_ReadCalibrationByte(PROD_SIGNATURES_START+ADCBCAL1_offset );
	}
}

// ========================= Flash  CAL ε ========================
// [μ] unsigned char index
// Item1: NVM address
// ----------------------------------------------------------------------------
unsigned char SP_ReadCalibrationByte(unsigned char index)
{
unsigned char flash *idx_base = (unsigned char*)0x0;	
unsigned char result;

	NVM.CMD = NVM_CMD_READ_CALIB_ROW_gc;			// Load the NVM Command register to read the calibration row.
	result = *(idx_base+index);
	
	NVM.CMD = NVM_CMD_NO_OPERATION_gc;			// Clean up NVM Command register.
	return result;
}

// =============================== ADC_Wait_8MHz ==============================
// [μ] ADC_t * adc
// Item1: ADC Port (ADCA,ADCB)
// ----------------------------------------------------------------------------
void ADC_Wait_8MHz(ADC_t *adc)
{
unsigned char prescaler_val = (adc)->PRESCALER;			// Store old prescaler value.

	(adc)->PRESCALER = ADC_PRESCALER_DIV4_gc;		// Set prescaler value to minimum value.
	delay_us(4*COMMON_MODE_CYCLES);			// Wait 4*COMMON_MODE_CYCLES for common mode to settle.
	(adc)->PRESCALER = prescaler_val;			// Set prescaler to old value
}

// =============================== ADC_Wait_32MHz =============================
// [μ] ADC_t * adc
// Item1: ADC Port (ADCA,ADCB)
// ----------------------------------------------------------------------------
void ADC_Wait_32MHz(ADC_t *adc)
{
unsigned char prescaler_val = (adc)->PRESCALER;			// Store old prescaler value.

	(adc)->PRESCALER = ADC_PRESCALER_DIV8_gc;		// Set prescaler value to minimum value.
	delay_us(8*COMMON_MODE_CYCLES);			// Wait 8*COMMON_MODE_CYCLES for common mode to settle.
	(adc)->PRESCALER = prescaler_val;			// Set prescaler to old value
}

// ========================== ADC Offset Get Signed ===========================
// [μ] ADC_CH_t *ch, bool oversampling
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// Item2: oversampling(false,TRUE)
// ----------------------------------------------------------------------------
char ADC_Offset_Get_Signed(ADC_CH_t *ch, bool oversampling)
{
char offset = 0;	
int i,offset1 = 0;

	if(oversampling){
		for(i = 0; i < 4; i++){
			ADC_Ch_Conversion_Start(ch);		// Do one conversion to find offset.
			while(!ADC_Ch_Conversion_Complete(ch));
			offset1 += ADC_ResultCh_GetWord_Signed(ch, 0x00);
		}
		return((char)(offset1 / 4));
	}
	else{        
		ADC_Ch_Conversion_Start(ch);			// Do one conversion to find offset.
		while(!ADC_Ch_Conversion_Complete(ch));
		offset = (char)ADC_ResultCh_GetWord_Signed(ch, 0x00);
		return(offset);
	}
}

// ========================= ADC Offset Get Unsigned ==========================
// [μ] ADC_CH_t *ch, bool oversampling
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// Item2: oversampling(false,TRUE)
// ----------------------------------------------------------------------------
unsigned char ADC_Offset_Get_Unsigned(ADC_CH_t *ch, bool oversampling)
{
unsigned char offset = 0;	
int i,offset1 = 0;

	if(oversampling){
		for(i = 0; i < 4; i++){
			ADC_Ch_Conversion_Start(ch);		// Do one conversion to find offset.
			while(!ADC_Ch_Conversion_Complete(ch));
			offset1 += ADC_ResultCh_GetWord_Signed(ch, 0x00);
		}
		return((unsigned char)(offset1 / 4));
	}
	else{        
		ADC_Ch_Conversion_Start(ch);			// Do one conversion to find offset.
		while(!ADC_Ch_Conversion_Complete(ch));
		offset = (unsigned char)ADC_ResultCh_GetWord_Signed(ch, 0x00);
		return(offset);
	}
}

// ====================== ADC ResultCh GetWord Unsigned =======================
// [μ] ADC_CH_t *adc_ch, unsigned char offset
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// Item2: offset 
// ----------------------------------------------------------------------------
unsigned int ADC_ResultCh_GetWord_Unsigned(ADC_CH_t *adc_ch, unsigned char offset)
{
unsigned int answer;

	(adc_ch)->INTFLAGS = ADC_CH_CHIF_bm;			// Clear interrupt flag
	answer = (adc_ch)->RES - offset;			// Return result register contents
	return(answer);
}

// ======================= ADC ResultCh GetWord Signed ========================
// [μ] ADC_CH_t *adc_ch, char signedOffset
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// Item2: offset 
// ----------------------------------------------------------------------------
int16_t ADC_ResultCh_GetWord_Signed(ADC_CH_t *adc_ch, char signedOffset)
{
int answer;


	(adc_ch)->INTFLAGS = ADC_CH_CHIF_bm;			// Clear interrupt flag
	answer = (adc_ch)->RES - signedOffset;			// Return result register contents
	return(answer);
}

// ========================== ADC ResultCh GetWord ============================
// [μ] ADC_CH_t *adc_ch
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// ----------------------------------------------------------------------------
unsigned int ADC_ResultCh_GetWord(ADC_CH_t *adc_ch)
{
	(adc_ch)->INTFLAGS = ADC_CH_CHIF_bm;			// Clear interrupt flag
	return((adc_ch)->RES);					// Return result register contents
}

// ========================= ADC ResultCh GetHighByte =========================
// [μ] ADC_CH_t *adc_ch
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// ----------------------------------------------------------------------------
unsigned char ADC_ResultCh_GetHighByte(ADC_CH_t *adc_ch)
{
	(adc_ch)->INTFLAGS = ADC_CH_CHIF_bm;			// Clear interrupt flag
	return((adc_ch)->RESH);					// Return result register contents
}

// ========================= ADC ResultCh GetLowByte ==========================
// [μ] ADC_CH_t *adc_ch
// Item1: ADC CH (ADCA.CH0, ADCA.CH1, ADCA.CH2, ADCA.CH3, ADCB.CH0, ADCB.CH1, ADCB.CH2, ADCB.CH3)
// ----------------------------------------------------------------------------
unsigned char ADC_ResultCh_GetLowByte(ADC_CH_t *adc_ch)
{
	(adc_ch)->INTFLAGS = ADC_CH_CHIF_bm;			// Clear interrupt flag
	return((adc_ch)->RESL);					// Return result register contents
}
