// ========================= AtxmegaxxxA1 DAC  α׷ ==================
//  CPU : ATmegaxxxA1
//     ¥ : 2010-
//      : Ver1.0
//    : www.cpuplaza.com
// Ϸ : CodeVisionAVR 2.04.6
// ----------------------------------------------------------------------------
// ⺻ 
#include <io.h>							// CPU  

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdbool.h>

//  
#include "dac_drv.h"						// DAC 

// ========================= DAC SingleChannel Enable =========================
// [μ] DAC_t *dac,DAC_REFSEL_t convRef, bool leftAdjust
// Item1: DAC Port (DACA,DACB)
// Item2: DAC Reference Selection(DAC_REFSEL_INT1V_gc,DAC_REFSEL_AVCC_gc,DAC_REFSEL_AREFA_gc,DAC_REFSEL_AREFB_gc)
// Item3: DAC Left-Adjust Value
// ----------------------------------------------------------------------------
void DAC_SingleChannel_Enable(DAC_t *dac,DAC_REFSEL_t convRef, bool leftAdjust)
{
	(dac)->CTRLB = ((dac)->CTRLB & ~DAC_CHSEL_gm ) | DAC_CHSEL_SINGLE_gc;
	(dac)->CTRLC = ((dac)->CTRLC & ~(DAC_REFSEL_gm | DAC_LEFTADJ_bm) ) | convRef | ( leftAdjust ? DAC_LEFTADJ_bm : 0x00 );
	(dac)->CTRLA = ((dac)->CTRLA & ~DAC_CH1EN_bm ) | DAC_CH0EN_bm | DAC_ENABLE_bm;
}

// ============================ DAC_DualChannel_Enable =========================
// [μ] DAC_t *dac, DAC_REFSEL_t convRef,bool leftAdjust,DAC_CONINTVAL_t sampleInterval,DAC_REFRESH_t refreshInterval
// Item1: DAC Port (DACA,DACB)
// Item2: DAC Reference Selection(DAC_REFSEL_INT1V_gc,DAC_REFSEL_AVCC_gc,DAC_REFSEL_AREFA_gc,DAC_REFSEL_AREFB_gc)
// Item3: DAC Left-Adjust Value
// Item4: DAC Timing Control Register(DAC_CONINTVAL_1CLK_gc,DAC_CONINTVAL_2CLK_gc,DAC_CONINTVAL_4CLK_gc,DAC_CONINTVAL_8CLK_gc
//                                    DAC_CONINTVAL_16CLK_gc,DAC_CONINTVAL_32CLK_gc,DAC_CONINTVAL_64CLK_gc,DAC_CONINTVAL_128CLK_gc
// ----------------------------------------------------------------------------
void DAC_DualChannel_Enable(DAC_t *dac, DAC_REFSEL_t convRef,bool leftAdjust,DAC_CONINTVAL_t sampleInterval,DAC_REFRESH_t refreshInterval)
{
	(dac)->CTRLB = ((dac)->CTRLB & ~DAC_CHSEL_gm ) | DAC_CHSEL_DUAL_gc;
	(dac)->CTRLC = ((dac)->CTRLC & ~( DAC_REFSEL_gm | DAC_LEFTADJ_bm ) ) | convRef | ( leftAdjust ? DAC_LEFTADJ_bm : 0x00 );
	(dac)->TIMCTRL = (uint8_t) sampleInterval | refreshInterval;
	(dac)->CTRLA |= DAC_CH1EN_bm | DAC_CH0EN_bm | DAC_ENABLE_bm;
}


// ============================ DAC Channel Write ============================
// [μ] DAC_t *dac,unsigned int data,DAC_CH_t channel
// Item1: DAC Port (DACA,DACB)
// Item2: DAC Out Data
// Item3: CH (CH0,CH1)
// ----------------------------------------------------------------------------
void DAC_Channel_Write(DAC_t *dac,unsigned int data,DAC_CH_t channel)
{
	if(channel == CH0){
		(dac)->CH0DATA = data;
	}
	else{
		(dac)->CH1DATA = data;
	}
}

// ============================ DAC Channel DataEmpty =========================
// [μ] DAC_t *dac, DAC_CH_t channel
// Item1: DAC Port (DACA,DACB)
// Item2: CH (CH0,CH1)
// ----------------------------------------------------------------------------
bool DAC_Channel_DataEmpty(DAC_t *dac,DAC_CH_t channel)
{
bool dacStatus;
	
	dacStatus = ((dac)->STATUS & (channel ? DAC_CH1DRE_bm : DAC_CH0DRE_bm));
	return(dacStatus);
}

// ============================ DAC EventAction Set ===========================
// [μ] DAC_t *dac,DAC_TRIG_t trigChannel,unsigned char eventLine
// Item1: DAC Port (DACA,DACB)
// Item2: trigChannel
// Item3: eventLine
// ----------------------------------------------------------------------------
void DAC_EventAction_Set(DAC_t *dac,DAC_TRIG_t trigChannel,unsigned char eventLine)
{
	(dac)->CTRLB = ((dac)->CTRLB & ~DAC_TRIG_1_1 ) | trigChannel;
	(dac)->EVCTRL = eventLine & DAC_EVSEL_gm;
}
