// =============================== EX_BUS  ============================
// ] atxmegaxxxA1 ø external bus 
//       * atmel ebi_driver ҽ
// ----------------------------------------------------------------------------
#ifndef __EBI_DRV_INCLUDED__
#define __EBI_DRV_INCLUDED__

// ũ 
/*! \brief Disable EBI module.
 *
 *  This function disables the EBI module.
 *  This will abort any on-going operations, such as DMA transfers.
 */
#define EBI_Disable() (EBI.CTRL = EBI.CTRL & ~EBI_IFMODE_gm | EBI_IFMODE_DISABLED_gc)



/*! \brief Disable a Chip Select module.
 *
 *  This function disables one Chip Select submodule.
 *
 * \param  _chipSelect  The EBI_CS_t struct instance
 */
#define EBI_DisableChipSelect(_chipSelect) ( (_chipSelect)->CTRLA = (_chipSelect)->CTRLA & ~EBI_CS_MODE_gm | EBI_CS_MODE_DISABLED_gc )



/*! \brief Switch between normal SDRAM mode and "Load Mode Register" operation.
 *
 *  This function selects normal operation or Load Mode Register for SDRAM operation.
 *  SDRAM must be enabled first.
 *
 *  \param  _sdramMode  Selects SDRAM mode.
 */
#define EBI_SetSDRAMMode(_sdramMode) ( EBI.CS3.CTRLB = EBI.CS3.CTRLB & ~EBI_CS_SDMODE_gm | (_sdramMode) )



/*!  \brief Enable SDRAM Self-refresh mode.
 *
 *  This function enables self-refresh for SDRAM. Use this before entering
 *  sleep mode so that SDRAM device refreshes itself. SDRAM will be inaccessible
 *  while in self-refresh mode.
 */
#define EBI_EnableSelfRefresh() (EBI.CS3.CTRLB |= EBI_CS_SDSREN_bm)



/*! \brief Disable SDRAM Self-refresh mode.
 *
 *  This function disables self-refresh for SDRAM. Use this after leaving
 *  sleep mode so that SDRAM is accessible again.
 */
#define EBI_DisableSelfRefresh() (EBI.CS3.CTRLB &= ~EBI_CS_SDSREN_bm)

// Լ 
void EBI_Enable(EBI_SDDATAW_t sdramDataWidth, EBI_LPCMODE_t lpcMode, EBI_SRMODE_t sramMode, EBI_IFMODE_t interfaceMode);
void EBI_EnableSRAM(EBI_CS_t *chipSelect,EBI_CS_ASPACE_t addrSpace,unsigned int baseAddr, unsigned char waitStateCycles);
void EBI_EnableLPC(volatile EBI_CS_t *chipSelect, EBI_CS_ASPACE_t addrSpace, unsigned int baseAddr, unsigned char waitStateCycles);
void EBI_EnableSDRAM( EBI_CS_ASPACE_t addrSpace, unsigned int baseAddr, bool casLatency, bool rowBits, EBI_SDCOL_t columnBits,
                      EBI_MRDLY_t modeRegisterDelay, EBI_ROWCYCDLY_t rowCycleDelay, EBI_RPDLY_t rowToPrechargeDelay, EBI_WRDLY_t writeRecoveryDelay,
                      EBI_ESRDLY_t exitSelfRefreshToActiveDelay, EBI_ROWCOLDLY_t rowToColumnDelay, unsigned int refreshPeriod, unsigned int initializationDelay);

#endif
